@@ -22,11 +22,12 @@ use Wireguard::WGmeta::Cli::Commands::Enable;
 use Wireguard::WGmeta::Cli::Commands::Disable;
 use Wireguard::WGmeta::Cli::Commands::Apply;
 use Wireguard::WGmeta::Cli::Commands::Add;
+use Wireguard::WGmeta::Cli::Commands::Remove;
 
 use base 'Exporter';
 our @EXPORT = qw(route_command);
 
-our $VERSION = "0.2.3";
+our $VERSION = "0.3.0";
 
 =head2 route_command($ref_list_input_args)
 
@@ -82,6 +83,10 @@ sub route_command($ref_list_input_args) {
             Wireguard::WGmeta::Cli::Commands::Add->new(@cmd_args)->entry_point();
             last;
         };
+        /^removepeer$/ && do {
+            Wireguard::WGmeta::Cli::Commands::Remove->new(@cmd_args)->entry_point();
+            last;
+        };
         /^apply$/ && do {
             Wireguard::WGmeta::Cli::Commands::Apply->new(@cmd_args)->entry_point();
             last;
