@@ -3,13 +3,15 @@ use strict;
 use warnings FATAL => 'all';
 use experimental 'signatures';
 use base 'Exporter';
-our @EXPORT = qw(prettify_message BOLD RESET GREEN RED YELLOW);
+our @EXPORT = qw(prettify_message BOLD RESET GREEN RED YELLOW GREEN_BACKGROUND RED_B);
 
 use constant BOLD => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[1m" : "";
 use constant RESET => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[0m" : "";
 use constant GREEN => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[32m" : "";
 use constant RED => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[31m" : "";
 use constant YELLOW => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[33m" : "";
+use constant GREEN_BACKGROUND => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[42m" : "";
+use constant RED_B => (!defined($ENV{'WG_NO_COLOR'})) ? "\e[41m" : "";
 
 sub prettify_message($error, $is_warning) {
     $error =~ s/at.*$//g unless (defined $ENV{IS_TESTING});
